/**
 * @file   shaft_encoder.c
 * @author Justin Tanner
 * @date   Tue Aug 26 18:31:33 2008
 * 
 * @brief  Hamamatsu 5587, P5588 ( or very similar ) Shaft Encoder
 *
 *         This driver initalizes the shaft encoder, and setups a timer to read
 *         high-low transitions on the output of the shaft encoder.
 *
 */

#include "shaft_encoder.h"

/** interrupt counter of timer1 used by the led controller */
extern uint16_t volatile interrupt_count;

/** actual line counter */
extern uint16_t volatile line_count;

/** count the number of rotations */
extern uint16_t volatile rotation_count;

/** value of the counter taken in the last interrupt */
uint16_t last_revolution = 0;

/** ticks @ 120000Hz for every rotation of the wheel */
uint16_t ticks_per_revolution = 0;

/** 
 * Initalize the shaft encoder and timer to capture its input
 * 
 */
void shaft_encoder_init(void)
{
    /* setup the input pin for the shaft encoder */
    SHAFT_DDR &= ~_BV(SHAFT_INPUT_PIN);
    
    /* noise canceller, running @ 125000Hz */
    TCCR3B |= _BV(ICNC3) | _BV(ICES3) | _BV(CS31) | _BV(CS30);

    /* set input capture on a pin, not waveform */
    TIFR3 |= _BV(ICF3);

    /* start input capture */
    TIMSK3 |= _BV(ICIE3);
}

/** 
 * Input capture timer firing on high-low transitions generated by the shaft encoder
 * This timer is responsible to adjusting the timers used by the led controller chips
 * 
 * @param TIMER3_CAPT_vect 
 * 
 */
ISR(TIMER3_CAPT_vect)
{
    LED_TIMER_OFF();

    /* calculate the ticks of the last revolution of the wheel */
    ticks_per_revolution = ICR3 - last_revolution;

    last_revolution = ICR3;

    /* get the ticks per line */
    OCR1A = ticks_per_revolution / LPR_ADJUSTED;    

    /* reset total interrupts counter for timer1 */
    interrupt_count = 0;
    line_count = 0;

    /* count the number of rotations */
    rotation_count++;

    LED_TIMER_ON();
}
